unit FNewCourse;

interface

uses
  SysUtils, Types, Classes, Variants, QTypes, QGraphics, QControls, QForms, 
  QDialogs, QStdCtrls, UStandard, UManageBase, QButtons, QExtCtrls,
  QComCtrls;

type
  TNewCourseForm = class(TForm)
    ExitButton: TBitBtn;
    NameEdit: TEdit;
    WidthRadio: TRadioGroup;
    Label7: TLabel;
    LengthRadio: TRadioGroup;
    Label8: TLabel;
    SpeedRadio: TRadioGroup;
    Label1: TLabel;
    ScaleRadio: TRadioGroup;
    Label2: TLabel;
    WarnPanel: TPanel;
    Progress: TProgressBar;
    CreateButton: TBitBtn;
    Label3: TLabel;
    Label4: TLabel;
    procedure ExitButtonClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure NameEditChange(Sender: TObject);
    procedure CreateButtonClick(Sender: TObject);
  private
    CourseName: String;   // course.tcl params
    CourseDir: String;
    CourseWidth: double;
    CourseLength: double;
    Playwidth: double;
    PlayLength: double;
    Scale: double;
    Angle: double;
    StartStr: String;

    Name: String;
    Done: boolean;
    function CheckName: boolean;
    procedure CreateCourse;
    procedure WriteCourseTCL;
  public
  end;

var
  NewCourseForm: TNewCourseForm;

implementation

{$R *.xfm}

procedure TNewCourseForm.CreateCourse; {-----------------------}
const bmw: array[0..4] of integer = (50,50,100,150,200);
      bmh: array[0..4] of integer = (500,500,1000,1500,2000);
      cw:  array[0..4] of integer = (50,100,200,300,400);
      ch:  array[0..4] of integer = (500,1000,2000,3000,4000);

var   bm: TBitmap;
      w, h, x, y: integer;
      fi: TStringList;
begin
  Progress.Position:=10;
  coursedir := iiShareDir + SEP + name;
  ForceDirectories (courseDir);

  bm:=TBitmap.Create;
  w := bmw [WidthRadio.ItemIndex];
  bm.Width := w;
  h := bmh [LengthRadio.ItemIndex];
  bm.Height := h;
  bm.Format := 'PNG';

  for x := 0 to w-1 do for y := 0 to h-1 do bm.Canvas.Pixels[x,y] := $00808080;
  bm.SaveToFile (coursedir + SEP + 'elev.png');
  Progress.Position := 50;
  for x := 0 to w-1 do for y := 0 to h-1 do bm.Canvas.Pixels [x,y] := clWhite;
  bm.SaveToFile (coursedir + SEP + 'terrain.png');
  Progress.Position:=90;
  bm.Free;

  CourseWidth := cw [WidthRadio.ItemIndex];
  CourseLength := ch [LengthRadio.ItemIndex];
  PlayLength:=CourseLength - 50;
  PlayWidth:=CourseWidth / 1.05;
  case SpeedRadio.ItemIndex of
    0: Angle := 15;
    1: Angle := 20;
    2: Angle := 25;
    3: Angle := 30;
    4: Angle := 35;
  end;
  case ScaleRadio.ItemIndex of
    0: Scale := 8;
    1: Scale := 16;
    2: Scale := 24;
    3: Scale := 32;
    4: Scale := 40;
  end;
  CourseName := Name;
  StartStr := '{' + IntToStr (w div 2) + ' 10}';
  WriteCourseTCL;
  CpFile (ProgDir + 'res' + SEP + 'preview.png', CourseDir + SEP + 'preview.png');
  Progress.Position := 100;
  WarnPanel.visible := true;
  WarnPanel.caption := ' ready - now you can create another course';
  NameEdit.SetFocus;
end;

procedure TNewCourseForm.WriteCourseTCL; {----------------------------------------}
var co:TStringList;
begin
  co:=TStringList.Create;
  co.Add('# generated by Tuxeditor');
  co.Add('');
  co.Add('objset :course -course_name "'+CourseName+'"');
  co.Add('objset :course:heightmap -width '+IntToStr(trunc(CourseWidth)));
  co.Add('objset :course:heightmap -length '+IntToStr(trunc(CourseLength)));
  co.Add('objset :course:heightmap -play_width '+IntToStr(trunc(PlayWidth)));
  co.Add('objset :course:heightmap -play_length '+IntToStr(trunc(PlayLength)));
  co.Add('objset :course:heightmap -angle '+IntToStr(trunc(Angle)));
  co.Add('objset :course:heightmap -elev_scale '+IntToStr(trunc(Scale)));
  co.Add('objcall :course:heightmap load_elevation elev.png');
  co.Add('');
  co.Add('objset :course -start_pt '+StartStr);
  co.Add('');
  co.Add('tux_course_init');
  co.SaveToFile(coursedir + SEP + 'course.tcl');
  co.Free;
end;

function TNewCourseForm.CheckName; {---------------------------}
var i, po: integer;
    oo: integer;
begin
  if length (NameEdit.Text) < 1 then begin
    Result := false;
    WarnPanel.visible := true;
    WarnPanel.caption := ' You must enter a course name';
    NameEdit.SetFocus;
  end else begin
    name := LowerCase (NameEdit.Text);
    repeat
      po := pos (' ', name);
      if po > 0 then name[po] := '_';
    until po = 0;
    for i := length (name) downto 1 do begin
      if not (name[i] in ['a'..'z', '_', '0'..'9']) then Delete (name, i, 1);
    end;
    if length (name) < 1 then begin
      Result := false;
      WarnPanel.visible := true;
      WarnPanel.caption := ' Not a valid ourse name';
      NameEdit.SetFocus;
    end else begin
      if FileExists (iiShareDir + SEP + name) then begin
        Result := false;
        WarnPanel.visible := true;
        WarnPanel.caption := ' Error: The course ' + name + ' exists already';
        NameEdit.SetFocus;
      end else begin
        Result := true;
      end;
    end;
  end;
end;

procedure TNewCourseForm.ExitButtonClick(Sender: TObject);
begin
  if Done then ModalResult := mrOK else ModalResult := mrCancel;
end;

procedure TNewCourseForm.FormShow(Sender: TObject);
begin
  WidthRadio.ItemIndex := 1;
  LengthRadio.ItemIndex := 1;
  SpeedRadio.ItemIndex := 2;
  ScaleRadio.ItemIndex := 2;
  NameEdit.Text := '';
  WarnPanel.visible := false;
  NameEdit.SetFocus;
  Done := false;
end;

procedure TNewCourseForm.NameEditChange(Sender: TObject);
begin
  WarnPanel.visible := false;
  Progress.Position := 0;
end;

procedure TNewCourseForm.CreateButtonClick(Sender: TObject);
begin
  if CheckName then begin
    CreateCourse;
    Done := true;
  end else begin
  end;;
end;

end.
