unit UManageBase;

interface

uses
  QGraphics, Types, SysUtils, Libc, Classes, Math, QDialogs, QExtCtrls,
  UStandard;

const
  orNames: array[1..18] of String =
     ('beginner1','beginner2','beginner3','arctic1','arctic2','arctic3',
      'forest1','forest2','forest3','mountain1','mountain2','mountain3',
      'glacier1','glacier2','glacier3','swiss1','swiss2','swiss3');
  orCups: array[1..18] of String =
     ('beginner','beginner','beginner','arctic','arctic','arctic',
      'forest','forest','forest','mountain','mountain','mountain',
      'glacier','glacier','glacier','swiss','swiss','swiss');

  orDirs: array[1..7] of String =
      ('beginner','arctic','forest','mountain','glacier','swiss','share');
  orLights: array[1..3] of String =
      ('sunny','sunset','foggy');
  orCubeView: array[1..6] of String =
      ('front','back','left','right','top','bottom');

  // herrings, time and conditions are defaults for usage in events
  orHer1: array[1..18] of integer =
     (16,20,18, 25,35,35, 25,52,18, 30,40,40, 30,28,25, 60,28,40);
  orHer2: array[1..18] of integer =
     (24,30,23, 36,56,50, 36,64,28, 38,56,50, 38,33,32, 70,40,45);
  orHer3: array[1..18] of integer =
     (25,31,24, 41,68,52, 40,70,32, 35,60,56, 44,35,35, 72,44,50);

  orTime1: array[1..18] of integer =
     (75,75,75, 140,165,160, 145,165,150, 190,185,210, 160,150,155, 190,190,165);
  orTime2: array[1..18] of integer =
     (58,55,57, 120,144,137, 120,140,130, 164,165,192, 139,132,136, 168,168,150);
  orTime3: array[1..18] of integer =
     (54,52,52, 116,140,123, 115,125,127, 145,155,187, 135,127,132, 162,160,146);

  orCond: array[1..18] of integer =
     (0,1,0, 0,0,2, 2,1,0, 0,2,1, 0,2,0, 0,2,1);
  orWind: array[1..18] of integer =
     (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
  orSnow: array[1..18] of integer =
     (0,0,1, 1,0,0, 0,0,0, 0,1,0, 1,0,0, 0,0,1);

  emNone   = 0;
  emStand  = 1;
  emAdd    = 2;

  tlSunny  = 1;
  tlSunset = 2;
  tlFoggy  = 3;

  vmFront  = 1;
  vmBack   = 2;
  vmLeft   = 3;
  vmRight  = 4;
  vmTop    = 5;
  vmBottom = 6;

  panSelect = 1;
  panEdit   = 2;
  panIntern = 3;
  panCover  = 4;
  panWork   = 5;
  panEnv    = 6;

var
  {--- directories ---}
  iiTuxDir: String;
  iiCourseDir: String;
  iiBackupDir: String;
  iiShareDir: String;
  iiObjectsDir: String;
//  iiEnvDir: String;
  iiBegDir : String;
  iiArcticDir: String;
  iiForestDir: String;
  iiMountDir: String;
  iiGlacierDir: String;
  iiSwissDir: String;
  iiResDir: String;
  iiConfDir: String;
  iiConfEx: boolean;


  {--- file pathes ---}
  iiTuxFile: String;   // binary of Tuxracer
  iiLangFile: String;  // language.tcl
  iiIdxFile: String;   // course_idx.tcl

  iiCourseDirOK: boolean;
  iiEventlistOK: boolean;
  iiPreparationOK: boolean;
  iiFilelistOK: boolean;
  iiBackupOK: boolean;
  iiEventMode: integer;           // see emConstants
  iiEventNum: integer;
  iiLightMode: integer;           // see tlConstants
  iiEnvMode: String;              // same as cup folder
  iiViewMode: integer;
  iiOrder: array[0..4] of integer; // event order
  iiLanguage: String;

  FileList: TStringList;   // copylist for objects and environments
  CourseList: TStringList; // all available courses
  EventList: array[0..4] of TStringList;
  EnvList: TStringList;    // environment list

  CourseIdx: String;       // verweist ber Namen auf Index in CourseList
  CourseNames: String;     // verweist ber Namen auf TrivName in EventList
  CourseDesc: String;      // verweist ber Namen auf Desc in Eventlist;
  CupNameStr: String;      // Liste der verwendeten Cup-Namen
  EnvIdx: String;          // Index auf EnvList - Format: EnvMode-LightMode
  CubeList: String;        // Liste der cubes, Format: EnvMode-LightMode

  FMarkStr: String;
  FMarkStruct: String;
  FMarkName: String;
  FMarkIdx: integer;
  FLangStr: String;

{--- envlist ---}
procedure B_SetEnvStr (s: String);
function  B_EnvStr: String;
function  B_Cube (dir: String): String;
procedure B_MakeEnvIdx;
function  B_ReadLightTCL (dir: String; light: integer): String;
procedure B_MakeEnvList;
function  B_LoadEnvList: boolean;
function  B_SaveEnvList: boolean;

{--- eventlist ---}
procedure B_ClearEvent (ev: integer);
procedure B_MakeEventList;
function  B_LoadEventList: boolean;
function  B_SaveEventList: boolean;
procedure B_MakeNameStr;

{--- courselist ---}
procedure B_ScanShareDir (mask: String);
procedure B_MakeCourseIdx;
procedure B_ReadCourseList (list: TStrings);
procedure B_MakeCourseList;

procedure B_MakePathes;
function  B_LoadConfig: boolean;
function  B_SaveConfig: boolean;

{-----------------} implementation {---------------------------}

{--------------------------------------------------------------}
{                   EnvList                                    }
{--------------------------------------------------------------}

procedure B_SetEnvStr; {---------------------------------------}
var idx: integer;
    ss: String;
begin
  ss := iiEnvMode + '-' + orLights [iiLightMode];
  idx := ParsInt (EnvIdx, ss, -1);
  if idx >= 0 then EnvList[idx] := s;
end;

function B_EnvStr; {-------------------------------------------}
var sss: String;
    idx: integer;
begin
  sss := iiEnvMode + '-' + orLights [iiLightMode];
  idx := ParsInt (EnvIdx, sss, -1);
  if idx >= 0 then Result := EnvList[idx] else Result := '';
end;

function B_Cube; {---------------------------------------------}
var sss: String;
begin
  sss := dir + '-' + orLights [iiLightMode];
  Result := ParsStr (CubeList, sss);
end;

procedure B_MakeEnvIdx; {--------------------------------------}
// and CubeList
var i: integer;
    ss, sss: String;
begin
  EnvIdx := '';
  if EnvList.count > 0 then begin
    for i := 0 to EnvList.count-1 do begin
      ss := EnvList[i];
      sss := ParsStr (ss, 'dir') + '-' + ParsStr (ss, 'light');
      EnvIdx := EnvIdx + '[' + sss + ']' + IntToStr (i);
    end;
  end;
  CubeList := '';
  CubeList := CubeList
    + '[beginner-sunny]beginnercube_[beginner-sunset]beginnerevening_[beginner-foggy]beginnerovercast_';
  CubeList := CubeList
    + '[arctic-sunny]arctic_[arctic-sunset]arctic_sunset_[arctic-foggy]arcticovercast_';
  CubeList := CubeList
    + '[forest-sunny]alpinesunny_[forest-sunset]alpinesunset_[forest-foggy]alpineovercast_';
  CubeList := CubeList
    + '[mountain-sunny]mountainsunny_[mountain-sunset]mountainsunset_[mountain-foggy]mountainovercast_';
  CubeList := CubeList
    + '[glacier-sunny]glaciersunny_[glacier-sunset]glaciersunset_[glacier-foggy]glacierovercast_';
  CubeList := CubeList
    + '[swiss-sunny]swisssunny_[swiss-sunset]swisssunset_[swiss-foggy]swissovercast_';
  CubeList := CubeList
    + '[common-sunny]sunny[common-sunset]night[common-foggy]cloudy';
end;

function B_ReadLightTCL; {-------------------------------------}
var r, g, b, i, po, p1, p2: integer;
    po1, po2, po3: double;
    src, cube, ss, aa, bb: String;
    res2: String;
    fi: TStringList;
begin
  if iiCourseDirOK = false then exit;
  res2 := '[struct]env';

  fi := TStringList.Create;
  case light of
    tlSunny: begin
      fi.LoadFromFile (iiCourseDir + SEP + dir + SEP + 'sunny_light.tcl');
      res2 := res2 + '[light]sunny[dir]' + dir;
      end;
    tlSunset: begin
      fi.LoadFromFile (iiCourseDir + SEP + dir + SEP + 'sunset_light.tcl');
      res2 := res2 + '[light]sunset[dir]' + dir;
      end;
    tlFoggy: begin
      fi.LoadFromFile (iiCourseDir + SEP + dir + SEP + 'foggy_light.tcl');
      res2 := res2 + '[light]foggy[dir]' + dir;
      end;
  end;

  // Aufbereitung der Liste
  for i := 0 to fi.Count-1 do fi[i] := BareStr (fi[i]);
  for i:= fi.Count-1 downto 1 do begin
    aa := fi[i-1];
    bb := fi[i];
    if (aa <> '') and (bb <> '') then begin
      if aa [length(aa)] = '\' then begin
        delete (aa, length(aa), 1);
        fi[i-1] := aa + bb;
        fi[i] := '';
      end;
    end;
  end;
  for i := fi.count-1 downto 0 do if fi[i] = '' then fi.delete(i);
  for i := 0 to fi.Count-1 do begin
    aa := fi[i];
    po := pos ('{-light0_ambient',aa);
    if po <> 0 then begin
      aa := copy (aa, po, 500);
      aa := changepart (aa, '{-light0_ambient}', '[ambient]');
      aa := changepart (aa, '{-light0_diffuse}', '[diffuse]');
      aa := changepart (aa, '{-light0_specular}', '[specular]');
      aa := changepart (aa, '{-light0_position}', '[position]');
      while pos ('{', aa) <> 0 do delete (aa, pos('{', aa), 1);
      while pos ('}', aa) <> 0 do delete (aa, pos('}', aa), 1);
      SeparateParams (aa, 'ambient');
      SeparateParams (aa, 'diffuse');
      SeparateParams (aa, 'specular');
      SeparateParams (aa, 'position');

      ss := ParsStr (aa, 'ambient');
      r := round (ParsFloat2 (ss, '1', 0) * 255);
      g := round (ParsFloat2 (ss, '2', 0) * 255);
      b := round (ParsFloat2 (ss, '3', 0) * 255);
      res2 := res2 + '[amb_r]' + IntToStr (r);
      res2 := res2 + '[amb_g]' + IntToStr (g);
      res2 := res2 + '[amb_b]' + IntToStr (b);

      ss := ParsStr (aa, 'diffuse');
      r := round (ParsFloat2 (ss, '1', 0) * 255);
      g := round (ParsFloat2 (ss, '2', 0) * 255);
      b := round (ParsFloat2 (ss, '3', 0) * 255);
      res2 := res2 + '[diff_r]' + IntToStr (r);
      res2 := res2 + '[diff_g]' + IntToStr (g);
      res2 := res2 + '[diff_b]' + IntToStr (b);

      ss := ParsStr (aa, 'specular');
      r := round (ParsFloat2 (ss, '1', 0) * 255);
      g := round (ParsFloat2 (ss, '2', 0) * 255);
      b := round (ParsFloat2 (ss, '3', 0) * 255);
      res2 := res2 + '[spec_r]' + IntToStr (r);
      res2 := res2 + '[spec_g]' + IntToStr (g);
      res2 := res2 + '[spec_b]' + IntToStr (b);

      ss := ParsStr (aa, 'position');
      po1 := ParsFloat2 (ss, '1', 0);
      po2 := ParsFloat2 (ss, '2', 0);
      po3 := ParsFloat2 (ss, '3', 0);
      res2 := res2 + '[pos_1]' + Float_Str (po1, 2);
      res2 := res2 + '[pos_2]' + Float_Str (po2, 2);
      res2 := res2 + '[pos_3]' + Float_Str (po3, 2);
    end;

    po := pos ('tux_fog', aa);
    if po <> 0 then begin
      p1 := pos ('{', aa);
      p2 := pos ('}', aa);
      ss := '[x]' + BareStr (copy (aa, p1+1, p2-p1-1));
      SeparateParams (ss, 'x');
      r := round (ParsFloat2 (ss, '1', 0) * 255);
      g := round (ParsFloat2 (ss, '2', 0) * 255);
      b := round (ParsFloat2 (ss, '3', 0) * 255);
      res2 := res2 + '[fog_r]' + IntToStr (r);
      res2 := res2 + '[fog_g]' + IntToStr (g);
      res2 := res2 + '[fog_b]' + IntToStr (b);
    end;
    po := pos ('tux_particle', aa);
    if po <> 0 then begin
      p1 := pos ('{',aa);
      p2 := pos ('}',aa);
      ss := '[x]' + BareStr (copy (aa, p1+1, p2-p1-1));
      SeparateParams (ss, 'x');
      r := round (ParsFloat2 (ss, '1', 0) * 255);
      g := round (ParsFloat2 (ss, '2', 0) * 255);
      b := round (ParsFloat2 (ss, '3', 0) * 255);
      res2 := res2 + '[part_r]' + IntToStr (r);
      res2 := res2 + '[part_g]' + IntToStr (g);
      res2 := res2 + '[part_b]' + IntToStr (b);
    end;

    po := pos ('{-filename}', aa);
    if po <> 0 then begin
      aa := copy (aa, po, 200);
      aa := changepart (aa, '{-filename}', '[filename]');
      while pos ('{', aa) <> 0 do delete (aa, pos('{',aa), 1);
      while pos ('}', aa) <> 0 do delete (aa, pos('}',aa), 1);
      ss := ParsStr (aa, 'filename');
      SeparateTCLPath (ss, aa, cube);
      res2 := res2 + '[cube]' + cube;
      if dir = 'share' then begin
        SeparateTCLPath (aa, bb, src);
        res2 := res2 + '[src]' + src;
      end else begin
        SeparateTCLPath (aa, bb, src);
        res2 := res2 + '[src]' + src;
      end;
    end;
  end;
  fi.Free;
  Result := res2 + '[changed]0';
end;

procedure B_MakeEnvList; {-------------------------------------}
var i: integer;
    dir, orig: String;
begin
  EnvList.Clear;
  for i := 1 to 7 do begin
    dir := orDirs[i];
    EnvList.Add (B_ReadLightTCL (dir, tlSunny));
    EnvList.Add (B_ReadLightTCL (dir, tlSunset));
    EnvList.Add (B_ReadLightTCL (dir, tlFoggy));
  end;
  EnvList.Add ('[struct]env[light]sunny[dir]common[src]common[cube]sunny');
  EnvList.Add ('[struct]env[light]sunset[dir]common[src]common[cube]night');
  EnvList.Add ('[struct]env[light]foggy[dir]common[src]common[cube]cloudy');
  orig := iiCourseDir + SEP + 'environment.ogl';
  if not FileExists (orig) then EnvList.SaveToFile (orig);
end;

function B_LoadEnvList; {------------------------------------}
begin
  if not iiCourseDirOK then exit;
  if FileExists (iiCourseDir + SEP + 'environment.lst') then begin
    EnvList.LoadFromFile (iiCourseDir + SEP + 'environment.lst');
    if EnvList.Count < 4 then B_MakeEnvList;
  end else begin
    B_MakeEnvList;
  end;
  B_MakeEnvIdx;
end;

function B_SaveEnvList; {------------------------------------}
begin
  if iiCourseDirOK = false then exit;
  EnvList.SaveToFile (iiCourseDir + SEP + 'environment.lst');
end;

{--------------------------------------------------------------}
{                   EventList                                  }
{--------------------------------------------------------------}

procedure B_ClearEvent; {--------------------------------------}
var i: integer;
begin
  for i := EventList[ev].count-1 downto 1 do EventList[ev].Delete (i);
end;

procedure B_MakeNameStr; {-------------------------------------}
var ev, i: integer;
    ss, struct: String;
begin
   CupNameStr := '';
   for ev := 0 to 4 do begin
     for i := 0 to EventList[ev].count-1 do begin
       ss := EventList[ev][i];
       struct := ParsStr (ss, 'struct');
       if struct = 'cup' then CupNameStr := CupNameStr + '-' + ParsStr (ss, 'name') + '-';
     end;
   end;
end;

procedure B_MakeEventList; {-----------------------------------}
var i, ev: integer;
    ss, name, trivname, desc: String;

procedure AddC {...............................................}
(list: integer; name: String; opp: integer);
var ss: String;
    i, idx: integer;
begin
  idx := ParsInt (CourseIdx, name, -1);
  if idx >= 0 then begin
    ss := CourseList[idx];
    if opp > 0 then SetParsInt (ss, 'opp', 1);
    ss := ss + '[ev]' + IntToStr (list);
    EventList[list].Add (ss);
  end;
end;

begin {........................................................}
  for i := 0 to 4 do EventList[i].Clear;
  // Solo Challenge:
  EventList[0].Add ('[struct]event[grp]orig[trl]orig[ev]0[name]solo_challenge'
    + '[trivname]Solo Challenge');
  EventList[0].Add ('[struct]cup[grp]orig[trl]orig[ev]0[name]beginner'
    + '[trivname]Beginner Cup');
  for i := 1 to 3 do AddC (0, 'beginner' + IntToStr (i), 0);
  EventList[0].Add ('[struct]cup[grp]orig[trl]orig[ev]0[name]arctic'
    + '[trivname]Arctic Cup');
  for i := 1 to 3 do AddC (0, 'arctic' + IntToStr (i), 0);
  EventList[0].Add ('[struct]cup[grp]orig[trl]orig[ev]0[name]forest'
    + '[trivname]Forest Cup');
  for i := 1 to 3 do AddC (0, 'forest' + IntToStr (i), 0);
  EventList[0].Add ('[struct]cup[grp]orig[trl]orig[ev]0[name]mountain'
    + '[trivname]Mountain Cup');
  for i := 1 to 3 do AddC (0, 'mountain' + IntToStr (i), 0);
  EventList[0].Add ('[struct]cup[grp]orig[trl]orig[ev]0[name]glacier'
    + '[trivname]Glacier Cup');
  for i := 1 to 3 do AddC (0, 'glacier' + IntToStr (i), 0);
  EventList[0].Add ('[struct]cup[grp]orig[trl]orig[ev]0[name]swiss'
    + '[trivname]Swiss Cup');
  AddC (0, 'swiss1', 0);
  AddC (0, 'swiss3', 0);
  AddC (0, 'swiss2', 0);

  EventList[1].Add ('[struct]event[grp]orig[trl]orig[ev]1[name]race_vs_opponents'
    + '[trivname]Race vs Opponents');
  EventList[1].Add ('[struct]cup[grp]orig[trl]orig[ev]1[name]beginner'
    + '[trivname]Beginner Cup');
  for i := 1 to 3 do AddC (1, 'beginner' + IntToStr (i), 1);
  EventList[1].Add ('[struct]cup[grp]orig[trl]orig[ev]1[name]arctic'
    + '[trivname]Arctic Cup');
  for i := 1 to 3 do AddC (1, 'arctic' + IntToStr (i), 1);
  EventList[1].Add ('[struct]cup[grp]orig[trl]orig[ev]1[name]forest'
    + '[trivname]Forest Cup');
  for i := 1 to 3 do AddC (1, 'forest' + IntToStr (i), 1);
  EventList[1].Add ('[struct]cup[grp]orig[trl]orig[ev]1[name]mountain'
    + '[trivname]Mountain Cup');
  for i := 1 to 3 do AddC (1, 'mountain' + IntToStr (i), 1);
  EventList[1].Add ('[struct]cup[grp]orig[trl]orig[ev]1[name]glacier'
    + '[trivname]Glacier Cup');
  for i := 1 to 3 do AddC (1, 'glacier' + IntToStr (i), 1);
  EventList[1].Add ('[struct]cup[grp]orig[trl]orig[ev]1[name]swiss'
    + '[trivname]Swiss Cup');
  AddC (1, 'swiss1', 1);
  AddC (1, 'swiss3', 1);
  AddC (1, 'swiss2', 1);

  EventList[2].Add ('[struct]event[grp]add[trl]add[ev]2[name]my_event_1'
    + '[trivname]My Event 1');
  EventList[3].Add ('[struct]event[grp]add[trl]add[ev]3[name]my_event_2'
    + '[trivname]My Event 2');
  EventList[4].Add ('[struct]event[grp]add[trl]add[ev]4[ev]0[name]my_event_3'
    + '[trivname]My Event 3');

  // make CourseNames and CourseDesc
  CourseNames := '[struct]names';
  CourseDesc := '[struct]desc';
  for ev := 0 to 4 do begin
    for i := 0 to EventList[ev].count-1 do begin
      ss := EventList[ev][i];
      if ParsStr (ss, 'struct') = 'course' then begin
        name := '[' + ParsStr (ss, 'name') + ']';
        if pos (name, CourseNames) = 0
        then CourseNames := CourseNames + name + ParsStr (ss, 'trivname');
        if pos (name, CourseDesc) = 0
        then CourseDesc := CourseDesc + name + ParsStr (ss, 'desc');
      end;
    end;
  end;
end;

function B_LoadEventList; {------------------------------------}
var fi: TStringList;
    i, ordr, ev: integer;
    ss, struct, grp: String;
begin
  if not iiCourseDirOK then exit;
  if FileExists (iiCourseDir + SEP + 'events.lst') then begin
    for i := 0 to 4 do EventList[i].Clear;
    fi := TStringList.Create;
    fi.LoadFromFile (iiCourseDir + SEP + 'events.lst');
    if fi.count > 0 then for i := 0 to fi.count-1 do begin
      ss := fi[i];
      struct := ParsStr (ss, 'struct');
      if struct = 'names' then begin
        CourseNames := ss;
      end else if struct = 'desc' then begin
        CourseDesc := ss;
      end else if struct = 'order' then begin
        for ordr := 0 to 4 do iiOrder[ordr] := ParsInt (ss, IntToStr(ordr), ordr);
      end else begin
        ev := ParsInt (ss, 'ev', -1);
        if (ev >= 0) and (ev < 5) then EventList[ev].Add (ss);
      end;
    end;
    fi.Free;
  end else begin
    B_MakeEventList;
  end;
  B_MakeNameStr;
  iiEventlistOK := true;
end;

function B_SaveEventList; {------------------------------------}
var fi: TStringList;
    ev, i: integer;
    ss: String;
begin
  if iiCourseDirOK = false then exit;
  fi := TStringList.Create;
  fi.Add (CourseNames);
  fi.Add (CourseDesc);
  ss := '[struct]order';
  for i := 0 to 4 do begin
    ss := ss + '[' + IntToStr(i) + ']' + IntToStr (iiOrder[i]);
  end;
  fi.Add (ss);
  for ev := 0 to 4 do begin
    if EventList[ev].count > 0
    then for i := 0 to EventList[ev].count-1 do fi.Add (EventList[ev][i]);
  end;
  fi.SaveToFile (iiCourseDir + SEP + 'events.lst');
  fi.Free;
end;

{--------------------------------------------------------------}
{                   CourseList                                 }
{--------------------------------------------------------------}

procedure B_ScanShareDir; {------------------------------------}
var sr: TSearchRec;
    li: TStringList;
    i: integer;
    path, ss: String;
begin
  li := TStringList.Create;
  if FindFirst (mask, faAnyFile, sr) = 0 then begin
    repeat
      if (sr.name <> '.') and (sr.name <> '..') then begin
        if (sr.attr and faAnyFile) > 0 then
        li.Add (sr.name);
      end;
    until FindNext (sr) <> 0;
    FindClose (sr);
  end;

  if li.count > 0 then for i := 0 to li.count-1 do begin
    path := iiShareDir + SEP + li[i] + SEP + 'course.tcl';
    if FileExists (path) then begin
      ss := '[struct]course[grp]add[trl]add';
      ss := ss + '[name]' + li[i] + '[cupdir]share';
      ss := ss + '[her1]0[her2]0[her3]0[time1]200[time2]200[time3]200';
      ss := ss + '[opp]0[mirr]no[cond]sunny[wind]no[snow]no';
      ss := ss + '[trivname]' + li[i] + '[desc]description of ' + li[i];
      if DirectoryExists (iiShareDir + SEP + li[i] + SEP + 'ai_targets')
        then ss := ss + '[opposs]1' else ss := ss + '[opposs]0';
      CourseList.Add (ss);
    end;
  end;
  li.Free;
end;

procedure B_MakeCourseIdx; {-----------------------------------}
var i: integer;
    ss, name, trivname: String;
begin
  CourseIdx := '';
  if CourseList.count > 0 then for i := 0 to CourseList.count-1 do begin
    ss := CourseList[i];
    name := ParsStr (ss, 'name');
    trivname := ParsStr (ss, 'trivname');
    CourseIdx := CourseIdx + '[' + name + ']' + IntToStr(i);
  end;
end;

procedure B_ReadCourseList; {----------------------------------}
var i: integer;
    ss: String;
begin
  list.clear;
  if CourseList.count > 0 then begin
    for i := 0 to CourseList.Count-1 do begin
      ss := CourseList[i];
      if ParsStr (ss, 'grp') = 'add' then list.Add (ParsStr (ss, 'name'));
    end;
    list.Add ('');
    for i := 0 to CourseList.Count-1 do begin
      ss := CourseList[i];
      if ParsStr (ss, 'grp') = 'orig' then list.Add (ParsStr (ss, 'name'));
    end;
  end;
end;

procedure B_MakeCourseList; {----------------------------------}
var po, i: integer;
    sss, name, ss: String;
    fi: TStringList;
begin
  CourseList.Clear;
  FLangStr := '';

  // laden und aufbereiten der bersetzungen
  if FileExists (iiTuxdir + SEP + 'language.tcl') then begin
    fi := TStringList.Create;
    fi.LoadFromFile (iiTuxDir + SEP + 'language.tcl');

    for i := fi.Count-1 downto 0 do begin
      if pos('tr_ls(' + iiLanguage, fi[i]) = 0 then fi.Delete (i);
    end;
    for i := 0 to fi.count-1 do begin
      ss := fi[i];
      po := pos (',', ss);
      if po <> 0 then ss := '[' + copy (ss, po+1, 255);
      po := pos (')', ss);
      if po <> 0 then ss[po] := ']';
      repeat
        po := pos ('"', ss);
        if po > 0 then Delete (ss, po, 1);
      until po = 0;
      FLangStr := FLangStr + ss;
    end;
    fi.Free;
  end;

  // first the standard courses
  for i := 1 to 18 do begin
    name := orNames[i];
    ss := '[struct]course[grp]orig[trl]orig[name]' + name;
    ss := ss + '[cupdir]' + orCups[i];
    ss := ss + '[her1]' + IntToStr (orHer1[i]);
    ss := ss + '[her2]' + IntToStr (orHer2[i]);
    ss := ss + '[her3]' + IntToStr (orHer3[i]);
    ss := ss + '[time1]' + IntToStr (orTime1[i]);
    ss := ss + '[time2]' + IntToStr (orTime2[i]);
    ss := ss + '[time3]' + IntToStr (orTime3[i]);

    ss := ss + '[opp]0[mirr]no';
    if orCond[i] = 0 then sss := 'sunny'
      else if orCond[i] = 1 then sss := 'cloudy'
      else sss := 'night';
    ss := ss + '[cond]' + sss;
    if orWind[i] = 0 then sss := 'no' else sss := 'yes';
    ss := ss + '[wind]' + sss;
    if orSnow[i] = 0 then sss := 'no' else sss := 'yes';
    ss := ss + '[snow]' + sss;

    ss := ss + '[trivname]' + ParsStr (FLangStr, name);
    ss := ss + '[desc]' + ParsStr (FLangStr, name + '_desc');
    ss := ss + '[opposs]1';
    CourseList.Add (ss);
  end;

  // now the additional courses
  if DirectoryExists (iiShareDir) then begin
    B_ScanShareDir (iiShareDir + SEP + '*');
  end;
  B_MakeCourseIdx;
end;

{--------------------------------------------------------------}
{                      Files                                   }
{--------------------------------------------------------------}

procedure B_MakePathes; {--------------------------------------}
var fi: TStringList;
    i: integer;
begin
  if iiCourseDirOK then begin
    iiTuxDir := ExtractFileDir (iiCourseDir);
    iiBackupDir := iiTuxDir + SEP + 'courses_sav';
    iiShareDir := iiCourseDir + SEP + 'share';
    iiObjectsDir := iiCourseDir + SEP + 'objects';
//    iiEnvDir := iiCourseDir + SEP + 'environment';
    iiBegDir := iiCourseDir + SEP + 'beginner';
    iiArcticDir := iiCourseDir + SEP + 'arctic';
    iiForestDir := iiCourseDir + SEP + 'forest';
    iiMountDir := iiCourseDir + SEP + 'mountain';
    iiGlacierDir := iiCourseDir + SEP + 'glacier';
    iiSwissDir := iiCourseDir + SEP + 'swiss';

    iiTuxFile := iiTuxDir + SEP + 'tuxracer';
    iiLangFile := iiTuxDir + SEP + 'language.tcl';
    iiIdxFile := iiCourseDir + SEP + 'course_idx.tcl';
  end;
end;

function B_LoadConfig; {---------------------------------------}
var fi: TStringList;
    i: integer;
begin
  fi := TStringList.Create;
  if FileExists (iiConfDir + SEP + 'manager.cfg') then begin
    fi.LoadFromFile (iiConfDir + SEP + 'manager.cfg');
    for i := 0 to fi.count-1 do begin
      if ParsStr(fi[i], 'struct') = 'common' then begin
        iiCourseDir := ParsStr (fi[i], 'coursedir');
        iiTuxFile := ParsStr (fi[i], 'tuxracer');
        iiLanguage := ParsStr (fi[i], 'lang');
      end;
    end;
    iiConfEx := true;
  end else begin
    iiCourseDir := '';
    iiTuxFile := '';
    iiLanguage := 'english';
    iiConfEx := false;
  end;
  fi.Free;
end;

function B_SaveConfig; {---------------------------------------}
var fi: TStringList;
    ss: String;
begin
  fi := TStringList.Create;
  ss := '[struct]common[coursedir]';
  if iiCourseDirOK then ss := ss + iiCourseDir;
  ss := ss + '[tuxracer]' + iiTuxFile;
  ss := ss + '[lang]' + iiLanguage;
  fi.Add (ss);
  fi.SaveToFile (iiConfDir + SEP + 'manager.cfg');
  fi.Free;
end;

end.



